from django.urls import path
from userextensions.views import (AddFavorite, DeleteFavorite, DeleteRecent, RefreshApiToken)


app_name = "userextensions"

urlpatterns = [
    # create views
    path('add_favorite/', AddFavorite.as_view(), name='add_favorite'),

    # delete views
    path('delete_favorite/<int:pk>', DeleteFavorite.as_view(), name='delete_favorite'),
    path('delete_recent/<int:pk>', DeleteRecent.as_view(), name='delete_recent'),

    # custom views
    path('refresh_api_token', RefreshApiToken.as_view(), name='refresh_api_token'),

]



"""

"""
from django.utils import timezone
from django.conf import settings
from django.urls import resolve
from urllib.parse import urlparse

# Use MiddlewareMixin when present (Django >= 1.10)
try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object

# import models
from userextensions.models import UserRecent


class UserRecentsMiddleware(MiddlewareMixin):
    """ add url to user recents on page request """

    def process_request(self, request):
        """ read uers and url (path) from request, if valid and not in a skip list, add to user's recents """
        # only track specified methods
        track_method_list = getattr(settings, 'TRACK_METHOD_LIST', ['GET'])
        if request.method not in track_method_list:
            return

        # do not track url if user is not authenticated
        if not request.user.is_authenticated:
            return
        if not getattr(request, 'user'):
            return

        # do not track if url (full path) can not be determined
        path = getattr(request, 'get_full_path')()
        if not path:
            return

        # do not track admin or debug urls
        skip_url_prefix_list = getattr(settings, 'SKIP_URL_PREFIX_LIST', ['/admin/', '/__debug__/'])
        for prefix in skip_url_prefix_list:
            if path.startswith(prefix):
                return

        # do not track urls in the skip_fixed_url_list
        skip_fixed_url_list = getattr(settings, 'SKIP_FIXED_URL_LIST', ['/', '/login/', '/logout/', ])
        for url in skip_fixed_url_list:
            if path == url:
                return

        # do not track invalid urls
        try:
            resolve(urlparse(path).path)
        except:
            return

        # add recent
        UserRecent.objects.update_or_create(url=request.get_full_path(),
                                            user=request.user,
                                            defaults=dict(url=request.get_full_path(),
                                                          user=request.user,
                                                          updated_at=timezone.now())
                                            )



"""

"""
from django.shortcuts import render, redirect
from django.contrib import messages
from django.utils import timezone
from django.views.generic import View, DeleteView
from rest_framework.authtoken.models import Token
from braces.views import LoginRequiredMixin

# import models
from userextensions.models import (UserRecent, UserFavorite)


class RefreshApiToken(LoginRequiredMixin, View):
    """ delete current user token and create a new one """
    def post(self, request):
        try:
            token = Token.objects.get(user=request.user)
            token.delete()
            Token.objects.get_or_create(user=request.user)
        except Exception as err:
            messages.add_message(request, messages.ERROR, "Could not complete requested action: {}".format(err),
                                 extra_tags='alert-danger')
        return redirect(self.request.META.get('HTTP_REFERER'))


class AddFavorite(LoginRequiredMixin, View):
    """ add (the current) url to the list of user favorites """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        name = self.request.POST.dict().get('name', None)

        # do not track url if user is not authenticated or user can not be determined
        if not request.user.is_authenticated:
            return redirect(referrer)
        user = getattr(request, 'user')
        if not user:
            return redirect(referrer)

        try:
            recent, is_new = UserFavorite.objects.get_or_create(url=referrer,
                                                                user=request.user,
                                                                defaults=dict(url=referrer,
                                                                              user=request.user,
                                                                              name=name,
                                                                              updated_at=timezone.now())
                                                                )
            if is_new:
                messages.add_message(request, messages.INFO, "Successfully added favorite",
                                     extra_tags='alert-info')
            else:
                messages.add_message(request, messages.INFO, "This url is already in your favorites",
                                     extra_tags='alert-info')
        except Exception as err:
            messages.add_message(request, messages.ERROR, "Error adding favorite: {}".format(err),
                                 extra_tags='alert-danger')
        return redirect(referrer)


class DeleteFavorite(LoginRequiredMixin, DeleteView):
    """ delete a favorite (by pk) and return to the referring page """
    def delete(self, request, *args, **kwargs):
        UserFavorite.objects.filter(**kwargs).delete()
        messages.add_message(self.request, messages.INFO, "Favorite successfully deleted", extra_tags='alert-info')
        return redirect(self.request.META.get('HTTP_REFERER'))


class DeleteRecent(LoginRequiredMixin, DeleteView):
    """ delete a recent (by pk) and return to the referring page """
    def delete(self, request, *args, **kwargs):
        UserRecent.objects.filter(**kwargs).delete()
        messages.add_message(self.request, messages.INFO, "Recent successfully deleted", extra_tags='alert-info')
        return redirect(self.request.META.get('HTTP_REFERER'))



import os
from setuptools import setup, find_packages
import userextensions

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

setup(
    name='django-userextensions',
    packages=find_packages(),
    include_package_data=True,
    version=userextensions.__version__,
    license=userextensions.__license__,
    author='David Slusser',
    author_email='dbslusser@gmail.com',
    description='A user extension module for django',
    #long_description=README,
    url='https://github.com/davidslusser/django-userextensions',
    download_url='https://github.com/davidslusser/django-userextensions/archive/0.0.2.tar.gz',
    keywords=['django', 'helpers', 'extension', 'user', 'profile'],
    classifiers=[],
    install_requires=[
        'django',
        'django-handyhelpers',
    ],
    dependency_links=[],
)
